/****************************************************************************
 *                                                                          *
 * This program called cvi (C verbal interpreter) looks for information     *
 * about variables and data types in an ANSI C program.                     *
 * Copyright (c) 1999 Chepelianskii Alexeii.                                *
 *                                                                          *
 * This program is free software; you can redistribute it and/or modify     *
 * it under the terms of the GNU General Public License as Published by     *
 * the Free Software Fondation; either version 2 of the License, or (at     *
 * your option) any later version.                                          *
 *                                                                          *
 * This program is distributed in the hope it will be useful, but WITHOUT   *
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or    *
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License     *
 * for more details.                                                        *
 *                                                                          *
 * You should have received a copy of the GNU General Public License along  *
 * with this program; if not, write to the Free Software Foundation, Inc.,  *
 * 675 Mass Ave, Cambridge, MA 02139, USA.                                  *
 *                                                                          *
 ****************************************************************************/



#include <stdio.h>
#include <stdlib.h>
#include "trad.h"

/* The base structure for the stack. */
struct List {
    void *area;
    struct List *prec;       /* precedent */
    struct List *next;
};

/* General code */
/* operations for a stack */
void stackInitG(list **l)
{
    *l = NULL;
}


void *pushG(int size, list **l)
{
    list *next;
    
    if ((next = (list *) malloc(sizeof(list))) == NULL ||
	(next->area = malloc(size)) == NULL)
       return NULL;
    next->prec = *l;
    if (*l != NULL)
       (*l)->next = next;
    next->next = NULL;
    *l = next;
    return next->area;
}

int setToFirstG(list **l)
{
    if (*l == NULL)
       return 0;
    for (; (*l)->prec ; *l = (*l)->prec)
       ;
    return 1;
}

int setToNextG(list **l)
{
    if (*l == NULL)
       return 0;
    (*l) = (*l)->next;
    return 1;
}

int setToPrevG(list **l)
{
    if (*l == NULL)
       return 0;
    (*l) = (*l)->prec;
    return 1;
}

int setToLastG(list **l)
{
    if (*l == NULL)
       return 0;
    for (; (*l)->next ; *l = (*l)->next)
       ;
    return 1;
}

void popG(list **l)
{
    list *temp;

    if (l == NULL) {
       fprintf(stderr, " popL : prec == NULL\n");
       return;
    }
    temp = *l;
    free((void *) (*l)->area);
    *l = (*l)->prec;
    if (*l && temp->next != NULL)
       (*l)->next = temp->next;
    else if (*l)
       (*l)->next = NULL;
    free((void *) temp);
}

void *getG(list *l)
{
    if (l != NULL)
       return l->area;
    return NULL;
}


void echG(list *l)
{
    void *temp;

    temp = l->area;
    l->area = l->prec->area;
    l->prec->area = temp;
}


int howManyG(list *l)
{
    int i;

    setToLastG(&l);
    for (i = 0; getG(l) != NULL ; l = l->prec)
       i++;
    return i;
}

void cleanG(list **l)
{
    while (*l != NULL)
       popG(l);
}

int isElemG(list *l)
{
    if (l != NULL)
       return 1;
    return 0;
}

    


